********************************
***     DEFINE CONSTANTS     ***
********************************

ADDR     EQU   $60        ; ADDRESS OF THE ESP32 SOFTCARD
P0ARD    EQU   $62        ; PORT0 ADDRESS TO SEND DATA
P1ARD    EQU   $64        ; PORT1 ADDRESS TO RECEIVE DATA
VAPPLE   EQU   84         ; APPLE VIDEO ON
VESP32   EQU   85         ; APPLE VIDEO OFF
KB       EQU   'K'        ; KEYBOARD BYTE
JX       EQU   'X'        ; JOYSTICK X BYTE
JY       EQU   'Y'        ; JOYSTICK Y BYTE
B1       EQU   '1'        ; JOYSTICK BUTTON 1 BYTE
B2       EQU   '2'        ; JOYSTICK BUTTON 2 BYTE
MX       EQU   'x'        ; MOUSE X BYTE
MY       EQU   'y'        ; MOUSE Y BYTE
MB       EQU   'b'        ; MOUSE BUTTON BYTE

RDY      EQU   6          ; INITIALIZED AND READY BYTE

PREAD    EQU   $FB1E      ; MONITOR PADDLE READ ROUTINE
HOME     EQU   $FC58      ; MONITOR HOME ROUTINE
PRHEX    EQU   $FDE3      ; MONITOR PRINT HEX CHAR ROUTINE
COUT     EQU   $FDF0      ; MONITOR PRINT CHAR ROUTINE
DOS      EQU   $3D0       ; DOS WARM ENTRY
LINPRT   EQU   $ED24      ; PRINT DECIMAL OF A,X
PRBLNK   EQU   $F948      ; PRINT 3 SPACES
PTR1     EQU   $06        ; 16-BIT POINTER IN ZERO PAGE
PTR2     EQU   $08        ; 16-BIT POINTER IN ZERO PAGE

*** Screenhole Equates
XL       =     $478       ; + MSLOT = LOW BYTE X-COORDINATE
YL       =     $4F8       ; + MSLOT = LOW BYTE Y-COORDINATE
XH       =     $578       ; + MSLOT = HIGH BYTE X-COORDINATE
YH       =     $5F8       ; + MSLOT = HIGH BYTE Y-COORDINATE
BUTTON   =     $778       ; + MSLOT = BUTTON STATUS

*** Offsets to Mouse Entry Points
SETMSE   =     $12
READMSE  =     $14
POSMSE   =     $16
CLAMPMSE =     $17
HOMEMSE  =     $18
INITMSE  =     $19

*** MOUSE CLAMING BOX = 2048 ($800)
MAXBOX_H EQU   $8         ; HIGH BYTE OF THE MAX CLAMPING BOX
MAXBOX_L EQU   $00        ; LOW BYTE OF THE MAX CLAMPING BOX

*** MIDPOINT OF MOUSE CLAMING BOX = 1024 ($400)
MIDBOX_H EQU   $4         ; HIGH BYTE OF THE CLAMPING BOX MID POINT
MIDBOX_L EQU   $00        ; LOW BYTE OF THE CLAMPING BOX MID POINT

********************************
***     DEFINE VARIABLES     ***
********************************

         ORG   $2FD0      ; VARIABLES START HERE
         JMP   START

SLOT     DB    $FF        ; SLOT OF THE ESP32 SOFTCARD
MSLOT    DB    $FF        ; SLOT OF THE MOUSE CARD
VID      DB    0          ; CURRENT VIDEO MODE
CANEXIT  DB    0          ; IF TRUE, THE PROGRAM CAN EXIT
XCURRENT DB    0          ; CURRENT X POSITION OF JOYSTICK
YCURRENT DB    0          ; CURRENT Y POSITION OF JOYSTICK
XLAST    DB    0          ; PREVIOUS X POSITION OF JOYSTICK
YLAST    DB    0          ; PREVIOUS Y POSITION OF JOYSTICK
L1       DB    0          ; PREVIOUS BUTTON 1 STATE
L2       DB    0          ; PREVIOUS BUTTON 2 STATE
C1       DB    0          ; CURRENT BUTTON 1 STATE
C2       DB    0          ; CURRENT BUTTON 2 STATE
KEY      DB    0          ; KEYBOARD CODE
S        DB    0          ; LOOP VARIABLE FOR PRINTING
SKIPX    DB    0          ; USED TO SKIP READING THE X-JOYSTICK
SKIPY    DB    128        ; USED TO SKIP READING THE Y-JOYSTICK
CN       DS    1,0        ; MOUSE X-reg setup
N0       DS    1,0        ; MOUSE Y-reg setup
MXCURR_L DB    MIDBOX_L   ; CURRENT MOUSE-X LOW BYTE
MXCURR_H DB    MIDBOX_H   ; CURRENT MOUSE-X HIGH BYTE
MYCURR_L DB    MIDBOX_L   ; CURRENT MOUSE-Y LOW BYTE
MYCURR_H DB    MIDBOX_H   ; CURRENT MOUSE-Y HIGH BYTE
MBCURR   DB    0          ; CURRENT MOUSE BUTTON STATE
MXLAST_L DB    0          ; CURRENT MOUSE-X LOW BYTE
MXLAST_H DB    0          ; CURRENT MOUSE-X HIGH BYTE
MYLAST_L DB    0          ; CURRENT MOUSE-Y LOW BYTE
MYLAST_H DB    0          ; CURRENT MOUSE-Y HIGH BYTE
MBLAST   DB    0          ; CURRENT MOUSE BUTTON STATE
MXDIRTY  DB    0          ; TRUE IF THE X-COORD HAS CHANGED
MYDIRTY  DB    0          ; TRUE IF THE Y-COORD HAS CHANGED
MSKIP    DB    0          ; USED TO SKIP READING THE MOUSE

         DS    $3000-*

*** 260 REM FIRST DETERMINE THE SLOT OF THE ESP32 SOFTCARD
*** 270 PRINT CHR$(13) + "CHECKING SLOT: ";

START    JSR   PRINT      ; PRINT STRING BELOW UNTIL 0
         HEX   8D
         ASC   "CHECKING SLOT: ",00

*** 280 FOR S = 0 TO 7
*** 290 ADDR = (3080 + S) * 16
*** 300 PRINT STR$(S) + "  ";
*** 310 VID = PEEK(ADDR + 1)
*** 320 IF VID = VAPPLE OR VID = VESP32
***         THEN SLOT = S : P0ARD = ADDR
*** 330 NEXT S

         LDA   #$C0       ; SET UP ADDR TO $C0XX
         STA   ADDR+1
         STA   P0ARD+1    ; AS WELL AS P0ARD
         STA   P1ARD+1    ; AND P1ARD
         LDX   #0         ; INIT LOOP

SRCH_LP  STX   S          ; AND SAVE IN S
         TXA
         PHA              ; SAVE RAW SLOT#
         JSR   PRHEX      ; PRINT SLOT NUMBER
         JSR   PRINT
         ASC   "  ",00

         PLA              ; RECOVER RAW NUMBER
         ASL              ; MULTIPLY BY 8
         ASL
         ASL
         ORA   #$80       ; ADD $80
         STA   ADDR
         LDY   #1
         LDA   (ADDR),Y
         STA   VID        ; AND SAVE IN VID
         CMP   #VAPPLE
         BEQ   FOUND
         CMP   #VESP32
         BEQ   FOUND

NEXT_S   INX              ;TRY NEXT SLOT
         CPX   #8
         BNE   SRCH_LP
         JMP   DONE_S

FOUND    STX   SLOT
         LDA   ADDR
         STA   P0ARD
         STA   P1ARD
         JMP   NEXT_S

*** 343 PRINT CHR$(13)
*** 350 IF P0ARD = 0 THEN PRINT "ESP32 CARD NOT FOUND" : END

DONE_S   LDA   #$8D       ; PRINT CR
         JSR   COUT
         LDX   SLOT
         CPX   #$FF       ; DID WE FIND BOARD?
         BNE   BOARDOK
         JSR   PRINT
         HEX   8D
         ASC   "ESP32 SOFTCARD NOT FOUND",8D00
         LDA   #$8D       ; PRINT CR
         JSR   COUT
         RTS

*** 360 P1ARD = P0ADR + 1 : REM PORT1 ADDRESS
*** 370 PRINT "ESP32 SOFTCARD FOUND IN SLOT " + STR$(SLOT)
*** 380 PRINT "PLEASE PLUG THE MONITOR THROUGH THE CARD"

BOARDOK  INC   P1ARD
         JSR   PRINT
         HEX   8D
         ASC   "ESP32 SOFTCARD FOUND IN SLOT ",00
         LDA   S
         JSR   PRHEX      ; PRINT SLOT NUMBER
         LDA   #$8D       ; PRINT CR
         JSR   COUT
         JSR   FIND_MS    ; FIND THE MOUSE CARD
         JSR   PRINT
         HEX   8D
         ASC   "PLEASE PLUG THE MONITOR THROUGH THE CARD",8D00

*** 390 FOR S = 0 TO 1000 : NEXT S
*** 400 POKE P0ARD, RDY : REM SEND READY TO THE CARD

         LDX   #25        ; WAIT FOR 2.5 SECONDS
         JSR   WAITX
         LDA   #RDY
         LDY   #0
         STA   (P0ARD),Y
         JSR   DELAY_S

*** 420 ;   REM MAIN LOOP
*** 430 VID = PEEK(P1ARD)
*** 440 IF VID = VAPPLE AND CANEXIT = 1 THEN HOME : END

MAINLP   LDY   #0
         LDA   (P1ARD),Y
         CMP   #VAPPLE
         BNE   KEY_PR
         LDA   CANEXIT
         BEQ   KEY_PR
         JSR   STOP_MS    ; STOP THE MOUSE IF FOUND
         JSR   HOME       ; CLEAR THE SCREEN
         RTS              ; EXIT THE PROGRAM


*** 450 KEY = PEEK(49152)
*** 460 IF KEY > 127 THEN GET DUMMY$ : POKE P0ARD, KB :
***      POKE P0ARD, KEY : CANEXIT = 1

KEY_PR   LDA   $C000      ; CHECK FOR KEYPRESS
         BPL   JOY_X
         PHA
         BIT   $C010      ; CLEAR STROBE
         LDA   #KB
         LDY   #0
         STA   (P0ARD),Y
         JSR   DELAY_S
         PLA
         STA   (P0ARD),Y
         JSR   DELAY_S
         LDA   #1
         STA   CANEXIT

*** 470 XCURRENT = PDL(0)
*** 480 IF XCURRENT <> XLAST THEN POKE P0ARD, JX :
***      POKE P0ARD, XCURRENT : XLAST = XCURRENT

JOY_X    INC   SKIPX      ; GET PDL(0) ONCE EVERY 128 ITERATIONS
         INC   SKIPX
         BNE   JOY_B1
         LDX   #0
         JSR   PREAD
         STY   XCURRENT
         CPY   XLAST
         BEQ   JOY_B1
         LDA   #JX
         LDY   #0
         STA   (P0ARD),Y
         JSR   DELAY_S
         LDA   XCURRENT
         STA   (P0ARD),Y
         JSR   DELAY_S
         STA   XLAST

*** 490 C1 = PEEK(49249)
*** 500 IF C1 <> L1 THEN POKE P0ARD, B1 :
***      POKE P0ARD, C1 : L1 = C1

JOY_B1   LDA   $C061      ; GET PB(0)
         AND   #$80       ; CLEAR THE UNIMPORTANT BITS
         CMP   L1         ; COMPARE WITH PREV STATE
         BEQ   JOY_B2     ; NO CHANGE
         STA   C1
         LDA   #B1
         LDY   #0
         STA   (P0ARD),Y
         JSR   DELAY_S
         LDA   C1
         STA   (P0ARD),Y
         JSR   DELAY_S
         STA   L1

*** 510 C2 = PEEK(49250)
*** 520 IF C2 <> L2 THEN POKE P0ARD, B2 :
***       POKE P0ARD, C2 : L2 = C2

JOY_B2   LDA   $C062      ; GET PB(1)
         AND   #$80       ; CLEAR THE UNIMPORTANT BITS
         CMP   L2         ; COMPARE WITH PREV STATE
         BEQ   JOY_Y      ; NO CHANGE
         STA   C2
         LDA   #B2
         LDY   #0
         STA   (P0ARD),Y
         JSR   DELAY_S
         LDA   C2
         STA   (P0ARD),Y
         JSR   DELAY_S
         STA   L2

*** 530 YCURRENT = PDL(1)
*** 540 IF YCURRENT <> YLAST THEN POKE P0ARD, JY :
***       POKE P0ARD, YCURRENT : YLAST = YCURRENT

JOY_Y    INC   SKIPY      ; GET PDL(1) ONCE EVERY 128 ITERATIONS
         INC   SKIPY
         BNE   MOUSE
         LDX   #1
         JSR   PREAD
         STY   YCURRENT
         CPY   YLAST
         BEQ   MOUSE
         LDA   #JY
         LDY   #0
         STA   (P0ARD),Y
         JSR   DELAY_S
         LDA   YCURRENT
         STA   (P0ARD),Y
         JSR   DELAY_S
         STA   YLAST

MOUSE    LDA   MSLOT
         BMI   ENDMAIN    ; NO MOUSE
         INC   MSKIP      ; GET ONCE EVERY 128 ITERATIONS
         INC   MSKIP
         BNE   ENDMAIN
         JSR   READ_MS    ; READ MOUSE COORDINATES AND BUTTON
         LDA   MXDIRTY
         BEQ   MOUSE_Y    ; NO X-COORD CHANGE
         LDA   #MX
         LDY   #0
         STA   (P0ARD),Y  ; SEND THE X-COORD HEADER
         JSR   DELAY_S
         LDA   MXCURR_H
         STA   (P0ARD),Y  ; SEND THE HIGH BYTE OF THE X-COORD
         JSR   DELAY_S
         LDA   MXCURR_L
         STA   (P0ARD),Y  ; SEND THE LOW BYTE OF THE X-COORD
         JSR   DELAY_S
         STY   MXDIRTY    ; CLEAR THE X-COORD DIRTY FLAG
MOUSE_Y  LDA   MYDIRTY
         BEQ   MOUSE_B    ; NO Y-COORD CHANGE
         LDA   #MY
         LDY   #0
         STA   (P0ARD),Y  ; SEND THE Y-COORD HEADER
         JSR   DELAY_S
         LDA   MYCURR_H
         STA   (P0ARD),Y  ; SEND THE HIGH BYTE OF THE Y-COORD
         JSR   DELAY_S
         LDA   MYCURR_L
         STA   (P0ARD),Y  ; SEND THE LOW BYTE OF THE X-COORD
         JSR   DELAY_S
         STY   MYDIRTY    ; CLEAR THE Y-COORD DIRTY FLAG
MOUSE_B  LDA   MBCURR
         CMP   MBLAST
         BEQ   ENDMAIN    ; NO BUTTON CHANGE
         LDA   #MB
         LDY   #0
         STA   (P0ARD),Y  ; SEND THE BUTTON HEADER
         JSR   DELAY_S
         LDA   MBCURR
         STA   (P0ARD),Y  ; SEND THE BUTTON STATE
         JSR   DELAY_S
         STA   MBLAST

*** 550 GOTO 430

ENDMAIN  JMP   MAINLP

*** SUBROUTINES ***

PRINT    PLA              ;GET POINTER TO STRING FROM STACK
         STA   PTR1
         PLA
         STA   PTR1+1
         BNE   INCPTR     ;WHICH IS ONE BYTE AFTER JSR

PRTCHAR  JSR   COUT       ;PRINT THE CHARACTER
INCPTR   INC   PTR1       ;GET NEXT CHARACTER
         BNE   SKP
         INC   PTR1+1
SKP      LDY   #0
         LDA   (PTR1),Y
         BNE   PRTCHAR    ;IF NOT 00, KEEP PRINTING
         LDA   PTR1+1     ;OTHERWISE, RESTORE STACK
         PHA
         LDA   PTR1
         PHA
         RTS              ;AND RETURN

DELAY_S  LDX   #10        ; DELAY BETWEEN SENDING CONSECUTIVE BYTES
DECR     DEX
         BNE   DECR
         RTS

WAIT100M LDA   #200       ; WAIT FOR 100.403 MILLISECONDS
         JSR   $FCA8
         RTS

WAITX    JSR   WAIT100M   ; WAIT FOR X TIMES 100 MILLISECONDS
         DEX
         BNE   WAITX
         RTS

****************************************
*** Call Mouse Firmware
****************************************
*** Entry conditions:
***    X - Cn
***    Y = n0
***    A = user defined
CALLFIRM PHA
         LDA   (PTR2),Y   ; Set low byte of Mouse
         STA   FIRMADR+1  ;    firmware routine
         LDX   CN         ; Entry X-reg
         LDY   N0         ; Entry Y-reg
         PLA              ; Entry A-reg
FIRMADR  JMP   $0000      ; Set by FIND_MS & CALLFIRM

*******************************************
*** Find and initialize the mouse firmware
*******************************************
*** Signature bytes of Mouse firmware:
***    Cn0C = $20
***    CnFB = $D6
FIND_MS  LDX   #8         ; Slot counter (+1)
         LDA   #0         ; Lo byte of Cn00
         STA   PTR2
         LDA   #$C8       ; Hi byte of Cn00 (+1)
         STA   PTR2+1
DEC_CN   DEC   PTR2+1     ; Decrement Cn
         DEX              ; Decrement slot counter
         BEQ   END_FIND   ; Mouse firmware not found
         LDY   #$C        ; Offset to Cn0C
         LDA   (PTR2),Y   ; Get byte
         CMP   #$20       ; Is it 1st ID byte?
         BNE   DEC_CN     ; No. Check next slot
         LDY   #$FB       ; Offset to CnFB
         LDA   (PTR2),Y   ; Get byte
         CMP   #$D6       ; Is it 2nd ID byte?
         BNE   DEC_CN     ; No. Check next slot
*** MOUSE FOUND
         LDA   PTR2+1
         STA   FIRMADR+2  ; Set hi byte of slot
         STA   CN         ; Save Cn tor X-reg
         ASL              ; Shift n to hi nibble
         ASL
         ASL
         ASL
         STA   N0         ; Save n0 tor Y-reg
         STX   MSLOT      ; Save slot
         JSR   PRINT
         HEX   8D
         ASC   "MOUSE CARD FOUND IN SLOT ",00
         LDA   MSLOT
         JSR   PRHEX      ; PRINT SLOT NUMBER
         LDA   #$8D       ; PRINT CR
         JSR   COUT
         JSR   START_MS
END_FIND RTS

*** START THE MOUSE
START_MS LDY   #INITMSE
         JSR   CALLFIRM   ; Initialize Mouse firmware
         LDY   #CLAMPMSE
         JSR   SETCLMP    ; Set new clamping values
         LDA   #0         ;    for X-coordinate 
         JSR   CALLFIRM   ; Clamp X-coordinate
         LDY   #CLAMPMSE
         JSR   SETCLMP    ; Set new clamping values
         LDA   #1         ;    for Y-coordinate
         JSR   CALLFIRM   ; Clamp Y-coordinate
         JSR   WRT_MS
         LDY   #SETMSE
         LDA   #1         ; Set passive mode
         JSR   CALLFIRM   ; Start mouse
         RTS

*** STOP THE MOUSE - CALLED ON EXIT
STOP_MS  LDA   MSLOT
         BMI   END_STOP   ; NO MOUSE
         LDY   #SETMSE
         LDA   #0         ; Stop the mouse
         JSR   CALLFIRM
END_STOP RTS

*** READ THE MOUSE X,Y,BUTTON AND UPDATE THE CURR AND LAST VARS
READ_MS  JSR   RLOVR_MS
         LDY   #READMSE
         JSR   CALLFIRM   ; Read Mouse position
         LDY   MSLOT
         LDA   XH,Y       ; Hi byte X-coordinate
         CMP   MXLAST_H
         BEQ   :1
         STA   MXCURR_H
         STA   MXLAST_H
         LDA   #1
         STA   MXDIRTY
:1       LDX   XL,Y       ; Lo byte X-coordinate
         CPX   MXLAST_L
         BEQ   :2
         STX   MXCURR_L
         STX   MXLAST_L
         LDA   #1
         STA   MXDIRTY
:2       LDA   YH,Y       ; Hi byte Y-coordinate
         CMP   MYLAST_H
         BEQ   :3
         STA   MYCURR_H
         STA   MYLAST_H
         LDA   #1
         STA   MYDIRTY
:3       LDX   YL,Y       ; Lo byte Y-coordinate
         CPX   MYLAST_L
         BEQ   :4
         STX   MYCURR_L
         STX   MYLAST_L
         LDA   #1
         STA   MYDIRTY
:4       LDA   BUTTON,Y   ; button status
         AND   #$80       ; CLEAR THE UNIMPORTANT BITS
         STA   MBCURR
         RTS

*** WRITES THE MOUSE CORRINDATES FROM THE VALUES STORED IN
*** THE MCURR VARS
WRT_MS   PHP
         SEI
         LDY   #POSMSE
         JSR   SETXYPOS
         JSR   CALLFIRM   ; Set the mouse position
         PLP
         RTS

*** ROLLS OVER THE MOUSE POSITON TO THE MIDDLE OF THE CLAMPBOX
*** IF ANY OF THE BOUNDARIES ARE HIT
RLOVR_MS JSR   CHKX_POS
         CPX   #1
         BNE   :5
         LDA   #MIDBOX_H
         STA   MXCURR_H
         LDA   #MIDBOX_L
         STA   MXCURR_L
         JSR   WRT_MS
:5       JSR   CHKY_POS
         CPX   #1
         BNE   :6
         LDA   #MIDBOX_H
         STA   MYCURR_H
         LDA   #MIDBOX_L
         STA   MYCURR_L
         JSR   WRT_MS
:6       RTS

*** CHECK IF THE X-COORDINATE HAS HIT A BOUNDARY
CHKX_POS LDX   #0         ; X REGISTER == 1 MEANS YES
         LDA   #0
         ORA   MXCURR_H
         ORA   MXCURR_L
         BEQ   CHKX_TRU   ; IF BOTH X ARE ZERO RETURN TRUE
         LDA   MXCURR_H
         CMP   #MAXBOX_H
         BNE   CHKX_FLS   ; IF THE HIGH BYTES ARE NOT EQUAL RETURN FALSE
         LDA   MXCURR_L
         CMP   #MAXBOX_L
         BNE   CHKX_FLS   ; IF THE LOW BYTES ARE NOT EQUAL RETURN FALSE
CHKX_TRU LDX   #1
CHKX_FLS RTS

*** CHECK IF THE Y-COORDINATE HAS HIT A BOUNDARY
CHKY_POS LDX   #0         ; X REGISTER == 1 MEANS YES
         LDA   #0
         ORA   MYCURR_H
         ORA   MYCURR_L
         BEQ   CHKY_TRU   ; IF BOTH Y ARE ZERO RETURN TRUE
         LDA   MYCURR_H
         CMP   #MAXBOX_H
         BNE   CHKY_FLS   ; IF THE HIGH BYTES ARE NOT EQUAL RETURN FALSE
         LDA   MYCURR_L
         CMP   #MAXBOX_L
         BNE   CHKY_FLS   ; IF THE LOW BYTES ARE NOT EQUAL RETURN FALSE
CHKY_TRU LDX   #1
CHKY_FLS RTS

****************************************
*** Set the Clamping Values
****************************************
*** Entry conditions:
***    XL/H = minimum value  
***    YL/H = maximum value
SETCLMP  LDA   #0         ; Min=0
         STA   XL
         STA   XH
         LDA   #MAXBOX_L
         STA   YL
         LDA   #MAXBOX_H
         STA   YH
         RTS

****************************************
*** Set a new mouse position
****************************************
*** Entry conditions:
***    MSLOT+XL/H = x-coordinate    
***    MSLOT+YL/H = y-coordinate  
SETXYPOS LDX   MSLOT
         LDA   MXCURR_L
         STA   XL,X
         LDA   MXCURR_H
         STA   XH,X
         LDA   MYCURR_L
         STA   YL,X
         LDA   MYCURR_H
         STA   YH,X
         RTS

*** PRINT A 16-BIT INTEGER STORED IN A,X FOLLOWED BY 3 SPACES
*PRINT_MS LDA   MXCURR_H
*         LDX   MXCURR_L
*         JSR   LINPRT
*         JSR   PRBLNK
*         RTS

*** A DIFFERENT WAY TO CLEAR THE KEYBOARD
*CLRKBD   LDA   #$00       ; LOAD THE ACC WITH 0
*         STA   $C000      ; CLEAR THE KEYBOARD
*         STA   $C010      ; CLEAR THE STROBE
*         RTS
